#!/usr/bin/env python

import os

import example_pythonpath
from example_utils import ExampleServer, ExampleStaticFile

from kombu import Exchange, Queue
from gevent import monkey

import mushroom
from mushroom import messaging
from mushroom import rpc

BROKER_URL = 'amqp://mushroom:mushroom@localhost//mushroom'
CHAT_EXCHANGE = Exchange('chatex', type='fanout')
CHAT_QUEUE = Queue('chat', exchange=CHAT_EXCHANGE, routing_key='chat')

class ChatServer(ExampleServer):
    urls = [
        ('/', ExampleStaticFile('chat.html')),
    ] + ExampleServer.urls

    def server_init(self):
        monkey.patch_socket()
        transport = messaging.Transport(BROKER_URL, CHAT_EXCHANGE, CHAT_QUEUE)
        self.mom = rpc.Engine(transport, mushroom.MethodDispatcher(self, prefix='mom_'))
        transport.start()

    def rpc_message(self, request):
        self.mom.notify('message', request.data, routing_key='chat')

    def mom_message(self, request):
        self.sessions.notify('message', request.data)


if __name__ == '__main__':
    ChatServer.main()



import logging
import socket

import gevent
import gevent.queue
from kombu import pools
from kombu import BrokerConnection

from mushroom.rpc import Engine
from mushroom.rpc import Message
from mushroom.rpc import Notification


logger = logging.getLogger('mushroom.messaging')


class Transport(object):

    def __init__(self, broker_url, exchange, queue):
        '''
        Create a transport capable of receiving from one queue and sending to
        multiple exchanges.

        Arguments:
        broker_url -- URL for connecting to the broker
        queue      -- kombu.Queue object to read messages from
        exchange   -- kombu.Exchange object to send messages to
        '''
        self.producer_pool = pools.producers[BrokerConnection(broker_url)]
        self.connection_pool = pools.connections[BrokerConnection(broker_url)]
        self.greenlet = None
        self.exchange = exchange
        self.queue = queue
        self.rpc_engine = None

    def start(self):
        assert self.rpc_engine, 'rpc_engine not set'
        assert not self.greenlet, 'already running'
        self.keep_running = True
        self.greenlet = gevent.spawn(self.mainloop)

    def stop(self, join=True):
        self.keep_running = False
        if self.greenlet and join:
            self.greenlet.join()

    def mainloop(self):
        try:
            with self.connection_pool.acquire() as connection:
                self.queue(connection.channel()).declare()
                with connection.Consumer(self.queue, callbacks=[self.callback]) as consumer:
                    while self.keep_running:
                        try:
                            connection.drain_events(timeout=1)
                        except socket.timeout:
                            continue
        finally:
            self.greenlet = None

    def callback(self, body, message):
        try:
            rpc_message = Message.from_list(body)
        except ValueError as e:
            # Message is malformed.
            logger.exception(e)
            message.reject()
        else:
            rpc_message.reply_to = message.properties.get('reply_to', '')
            self.rpc_engine.handle_message(rpc_message)
            message.ack()

    def send(self, message, routing_key=None):
        if not routing_key:
            # No routing key means that we are sending a notification or
            # response. So we need to extract the reply_to information
            # from the request object.
            request = message.request
            routing_key = request.reply_to
        with self.producer_pool.acquire() as producer:
            producer.maybe_declare(self.exchange)
            producer.publish(message.to_list(), routing_key=routing_key,
                    reply_to=self.queue.routing_key)


class Client(Engine):

    def __init__(self, broker_url, exchange, queue, rpc_handler):
        super(Client, self).__init__(
                Transport(broker_url, exchange, queue),
                rpc_handler)

    def start(self):
        self.transport.start()

    def stop(self):
        self.transport.stop()



#!/usr/bin/env python

from time import time

import gevent

import example_pythonpath
from example_utils import ExampleServer, ExampleStaticFile


class TimePusherServer(ExampleServer):
    urls = [
        ('/', ExampleStaticFile('time-pusher.html')),
    ] + ExampleServer.urls

    def server_init(self):
        gevent.spawn(self.time_loop)

    def time_loop(self):
        while True:
            gevent.sleep(1)
            self.sessions.notify('time', time())


if __name__ == '__main__':
    TimePusherServer.main()



from gevent.event import Event

from mushroom.rpc import Heartbeat
from mushroom.rpc import Disconnect
from mushroom.rpc import Message
from mushroom.utils import Observable


class UnreliableTransport(object):
    '''
    Base class for unreliable transports. Unreliable means that
    the underlying protocol does not guarantee message delivery
    by itself. This is typically the case for any protocol that
    does raw communication without a layer that guarantees message
    delivery by itself.
    '''

    def __init__(self):
        # Outbound messages not yet acknowledged by the remote side
        self.messages = []
        # Last received message id. This is used to drop messages
        # already received from the remote side.
        self.last_message_id = None
        # Handler for RPC messages
        self.rpc_engine = None
        # Remote ip address; set by the transport implementation.
        self.remote_addr = None
        # Transport state
        self.state = Observable('DISCONNECTED')

    def handle_message(self, message):
        if hasattr(message, 'message_id'):
            if self.last_message_id is not None and \
                    message.message_id <= self.last_message_id:
                # Ignore messages that we have already received
                return
            else:
                self.last_message_id = message.message_id
        if isinstance(message, Heartbeat):
            self.handle_heartbeat(message)
        elif isinstance(message, Disconnect):
            self.handle_disconnect()
        else:
            # All messages except for Heartbeat and Disconnect
            # are handled by the RPC engine.
            self.rpc_engine.handle_message(message)

    def handle_heartbeat(self, heartbeat):
        last_message_id = heartbeat.last_message_id
        if last_message_id is not None:
            index = 0
            for message in self.messages:
                if message.message_id <= last_message_id:
                    index += 1
                else:
                    break
            if index > 0:
                del self.messages[:index]

    def handle_connect(self):
        # The client managed to connect. Try resending messages which
        # have not been acknowledged, yet.
        self.state('CONNECTED')
        for message in self.messages:
            self.real_send(message)

    def handle_disconnect(self, reconnect=False):
        if reconnect:
            self.state('CONNECTING')
        else:
            self.state('DISCONNECTED')

    def connect(self):
        '''
        Start transport and connect to the remote side. This method
        is transport specific and must be overwriten.
        '''
        raise NotImplementedError

    def disconnect(self):
        '''
        Disconnect from the remote side and stop transport. This method
        is transport specific and must be overwritten.
        '''
        raise NotImplementedError

    def send(self, message):
        self.messages.append(message)
        if self.state() == 'CONNECTED':
            self.real_send(message)

    def real_send(self, message):
        '''
        Perform the actual send operation. This method is only called
        internally and should not be called from application code.
        This method is transport specific and must be overwritten.
        '''
        raise NotImplementedError



import argparse
import codecs
import logging

from mushroom.application import Application
from mushroom.http import HttpResponse
from mushroom.rpc import MethodDispatcher
from mushroom.server import Server
from mushroom.session import SessionHandlerAdapter


class StaticFile(object):
    content_types = {
        'html': 'text/html',
        'txt': 'text/plain',
        'css': 'text/css',
        'js': 'text/javascript',
        'json': 'application/json',
        'jpg': 'image/jpeg',
        'png': 'image/png',
        'gif': 'image/gif',
        'ico': 'image/x-icon',
    }

    def __init__(self, filename):
        self.filename = filename
        try:
            name, extension = filename.rsplit('.', 1)
        except ValueError:
            self.content_type = 'application/octet-stream'
        else:
            self.content_type = self.content_types.get(extension,
                    'application/octet-stream')

    def load_file(self):
        with codecs.open(self.filename) as fh:
            return fh.read()

    def get(self, request):
        return HttpResponse('200 OK', self.load_file(),
                extra_headers={'Content-Type': self.content_type})


def parse_listener(value):
    # This code raises a ValueError if the wrong listener
    # format is used. The `argparse` module converts those
    # to the proper `ArgumentError` exceptions.
    host, port = value.split(':')
    port = int(port)
    if not 0 < port < 65536:
        raise ValueError
    return (host, port)
parse_listener.__name__ = 'host:port'


class SimpleApplication(Application):

    def __init__(self, urls=None, rpc_handler=None, session_handler=None):
        self.urls = urls or []
        super(SimpleApplication, self).__init__(rpc_handler, session_handler)

    def request(self, request):
        websocket = request.environ.get('wsgi.websocket')
        if request.method == 'GET' and not websocket:
            # FIXME The current mushroom version still uses a splitted
            # path which is not very pretty. When request.path is a normal
            # string again this code must be removed.
            request_path = '/' + '/'.join(request.path)
            for path, resource in self.urls:
                if request_path == path:
                    return resource.get(request)
            return HttpResponse('404 Not Found', '404 Not Found',
                    extra_headers={'Content-Type': 'text/plain'})
        return super(SimpleApplication, self).request(request)


class SimpleServer(Server):
    '''
    This is the preferred way of starting to work with mushroom.
    This server class makes it possible to use mushroom with only
    a few lines of code by relying on some defaults:

        - All RPC methods are prefixed with ``rpc_``
        - Session handlers calls are prefixed wit ``session_``
        - The server runs on localhost with port ``39288``

    In order to use this class create a subclass from it, overwrite
    the urls attribute and start `YourServer.main()`.
    '''
    urls = None
    host = '127.0.0.1'
    port = 39288 # picked by random.randint(1024, 65535)

    def __init__(self, listener, **kwargs):
        application = SimpleApplication(
                urls=self.urls,
                rpc_handler=MethodDispatcher(self, 'rpc_'),
                session_handler=SessionHandlerAdapter(self, 'session_'))
        super(SimpleServer, self).__init__(listener,
                application=application,
                **kwargs)
        self.server_init()

    def server_init(self):
        pass

    def session_authenticate(self, session, auth):
        return True

    def session_connect(self, session):
        pass

    def session_disconnect(self, session):
        pass

    @classmethod
    def main(cls):
        parser = argparse.ArgumentParser()
        parser.add_argument('listener',
                metavar='listener',
                nargs='?',
                default='%s:%d' % (cls.host, cls.port),
                type=parse_listener,
                help='Host and port to start http server on (default: %s:%d)' % (cls.host, cls.port))
        args = parser.parse_args()
        logging.basicConfig()
        print('Server running at http://%s:%d/' % args.listener)
        server = cls(args.listener)
        server.run()



class RpcError(RuntimeError):
    '''
    Base class for all exceptions raised from by the `Engine`.
    '''
    def __init__(self, message=''):
        self.message = message
        super(RpcError, self).__init__(message)


class MethodNotFound(RpcError):
    '''
    This error is raised when a method for a `Request` or `Notification`
    message is not found. This can either happen when a connected client
    tries to call a server method or the server tries to call a method
    on the client side.
    '''
    def __init__(self, method_name):
        self.method_name = method_name
        super(MethodNotFound, self).__init__(method_name)


class RequestTimeout(RpcError):
    '''
    This error is raised when a `Request` message is not answered within
    a specified timeout value. By default the value is set to infinite
    can be set do a different value when making the request.
    '''
    pass


class RequestException(RpcError):
    '''
    This exception is raised when a `Request` message is answered with
    an `Error` message.
    '''

    def __init__(self, data):
        super(RequestException, self).__init__()
        self.data = data


class RequestValidationError(RequestException):
    # FIXME
    pass



from gevent.pywsgi import WSGIServer
from geventwebsocket.handler import WebSocketHandler


class Server(WSGIServer):

    def __init__(self, listener, *args, **kwargs):
        kwargs.setdefault('handler_class', WebSocketHandler)
        super(Server, self).__init__(listener, *args, **kwargs)

    @property
    def sessions(self):
        return self.application.sessions

    def run(self):
        self.serve_forever()



#!/usr/bin/env python

from __future__ import absolute_import

import os
import sys

SERVER_HOST = '127.0.0.1'
SERVER_PORT = 39288 # picked by random.randint(1024, 65535)

TESTS_DIR = os.path.dirname(os.path.abspath(__file__))
PROJECT_ROOT = os.path.dirname(os.path.dirname(TESTS_DIR))
SELENIUM_HTML = os.path.join(TESTS_DIR, 'selenium.html')

if __name__ == '__main__':
    del sys.path[0]
    sys.path[0:0] = [PROJECT_ROOT]

import gevent
from gevent.event import Event
from gevent import subprocess
import json
import logging
from selenium import webdriver
import signal
import six
import threading
import time
import unittest

import mushroom
from mushroom.tests.utils import TestServer
from mushroom.http import WebSocketTransport, PollTransport



class AsyncDict(object):
    '''
    Dictionary like container that supports waiting for keys.
    '''

    def __init__(self):
        self.d = {}
        self.events = []

    def _notify_all(self):
        for event in self.events:
            event.set()

    def update(self, d):
        self.d.update(d)
        self._notify_all()

    def __setitem__(self, name):
        self.d['name'] = value
        self._notify_all()

    def __getitem__(self, name):
        return self.d['name']

    def get(self, name, timeout=None, clear=False):
        event = Event()
        try:
            self.events.append(event)
            with gevent.Timeout(timeout):
                while True:
                    try:
                        if clear:
                            return self.d.pop(name)
                        else:
                            return self.d[name]
                    except KeyError:
                        event.wait()
                        event.clear()
        finally:
            self.events.remove(event)



class ServerControl(object):
    '''
    Wrapper around the server subprocess which provides a way to receive
    updates of the async state_dict via stdout. The updates are JSON
    serialized and every line on stdout contains exactly one dictionary
    that is used to update the state_dict.
    '''

    def __init__(self):
        self.state_dict = AsyncDict()

    def start(self):
        self.proc = subprocess.Popen(
                [sys.executable, os.path.abspath(__file__)],
                stdout=subprocess.PIPE,
                env={
                    'PYTHONDONTWRITEBYTECODE': '1'
                })
        gevent.spawn(self.read_from_stdout)
        # wait for server to be online
        self.get('online', timeout=5)

    def stop(self):
        self.proc.terminate()
        self.proc.wait()

    def read_from_stdout(self):
        while True:
            line = self.proc.stdout.readline()
            if not line:
                return
            d = json.loads(line)
            self.state_dict.update(d)

    def get(self, name, timeout=None, clear=False):
        '''
        Retrieve state variable from state dict. If the variable
        is not set wait for it `timeout` seconds.
        '''
        return self.state_dict.get(name, timeout=timeout, clear=clear)


class SeleniumTestServer(TestServer):
    '''
    Server which is being controlled by the ServerControl. This class
    provides a `set` method which produces output on stdout which is
    then parsed in the ServerControl class to update the state_dict.
    This is used by the test cases to report if the expected methods
    were called in the correct way.
    '''

    def __init__(self, listener):
        super(SeleniumTestServer, self).__init__(listener,
                SELENIUM_HTML,
                rpc_handler=mushroom.MethodDispatcher(self, 'rpc_'),
                session_handler=mushroom.SessionHandlerAdapter(self, 'session_'),
                log=None)
        self.set_sessions()

    def set(self, name, value):
        sys.stdout.write(json.dumps({ name: value }))
        sys.stdout.write('\n')
        sys.stdout.flush()

    def set_sessions(self):
        self.set('sessions', [
            {
                'id': session.id,
                'transport': session.transport.name
            }
            for session in self.sessions
        ])

    def session_authenticate(self, session, auth):
        return True

    def session_connect(self, session):
        self.set_sessions()

    def session_disconnect(self, session):
        self.set_sessions()

    def rpc_message(self, request):
        self.sessions.notify('message', request.data)

    def rpc_test_request(self, request):
        self.set('request_received', True)
        return 42

    def rpc_test_notification(self, request):
        self.set('notification_received', True)


class SeleniumTestCase(object):

    @classmethod
    def setUpClass(cls):
        # remove http(s)_proxy environment variables as they can
        # screw up the tests.
        cls.env_backup = {
            'http_proxy': os.environ.pop('http_proxy', None),
            'https_proxy': os.environ.pop('https_proxy', None)
        }
        cls.server_url = 'http://%s:%s/' % (SERVER_HOST, SERVER_PORT)
        cls.browser = webdriver.Firefox()
        cls.browser.set_window_position(0, 0);
        cls.browser.set_window_size(600, 400);
        cls.browser.set_script_timeout(1)

    @classmethod
    def tearDownClass(cls):
        if hasattr(cls, 'browser'):
            cls.browser.quit()
        # restore environment variables
        for name, value in six.iteritems(cls.env_backup):
            if value:
                os.environ[name] = value

    def setUp(self):
        self.server = ServerControl()
        self.server.start()
        self.browser.get(self.server_url)
        self.browser.execute_script('set_transport("%s")' % self.transport)

    def tearDown(self):
        self.server.stop()

    def browser_exec(self, func):
        func_type = self.browser.execute_script('typeof(%s)' % func);
        if (func_type == 'undefined'):
            raise RuntimeError('typeof(%s) === undefined' % func)
        try:
            js = '%s(arguments[0])' % func;
            result = self.browser.execute_async_script(js)
        finally:
            js_error = self.browser.execute_script('return jsError;');
        if js_error:
            raise RuntimeError('JavaScript error: %s' % js_error)
        return result

    def test_connect(self):
        self.assertEqual(len(self.server.get('sessions', timeout=1, clear=True)), 0)
        self.assertTrue(self.browser_exec('test_connect'));
        sessions = self.server.get('sessions', timeout=1, clear=True)
        self.assertEqual(len(sessions), 1)
        self.assertEqual(sessions[0]['transport'], self.transport)

    def test_connect_server_offline(self):
        # This test assumes that no service is running at the highest
        # available port number (65535). If you are running a service
        # at this port number this test will fail.
        self.assertTrue(self.browser_exec('test_connect_server_offline'));

    def test_send_request(self):
        self.assertIs(self.browser_exec('test_send_request'), True);
        self.assertTrue(self.server.get('request_received', timeout=1))

    def test_send_notification(self):
        self.assertIs(self.browser_exec('test_send_notification'), True);
        self.assertTrue(self.server.get('notification_received', timeout=1))

    def test_send_disconnect(self):
        self.assertEqual(len(self.server.get('sessions', timeout=1, clear=True)), 0)
        self.assertIs(self.browser_exec('test_disconnect_1'), True)
        self.assertEqual(len(self.server.get('sessions', timeout=1, clear=True)), 1)
        self.assertIs(self.browser_exec('test_disconnect_2'), True)
        self.assertEqual(len(self.server.get('sessions', timeout=1, clear=True)), 0)


class WebSocketSeleniumTestCase(SeleniumTestCase, unittest.TestCase):
    transport = 'ws'

    def __init__(self, *args, **kwargs):
        unittest.TestCase.__init__(self, *args, **kwargs)


class PollSeleniumTestCase(SeleniumTestCase, unittest.TestCase):
    transport = 'poll'

    def __init__(self, *args, **kwargs):
        unittest.TestCase.__init__(self, *args, **kwargs)


if __name__ == '__main__':
    logging.basicConfig()
    server = SeleniumTestServer((SERVER_HOST, SERVER_PORT))
    server.start()
    server.set('online', True)
    server.serve_forever()



from gevent.event import AsyncResult


class Message(object):
    '''
    Base class for all messages.
    '''

    @staticmethod
    def from_list(l):
        '''
        Parse a list as defined in the protocol into a message object.

        :param l: list to be parsed
        '''
        if not isinstance(l, (list, tuple)):
            raise ValueError('Message is not encoded as list or tuple')
        try:
            message_class = MESSAGE_CLASS_BY_CODE[l[0]]
        except KeyError:
            raise ValueError('Unsupported message code: %r' % l[0])
        message = message_class.from_list(l)
        return message


class Heartbeat(Message):
    '''
    Heartbeat message
    '''
    code = 0

    def __init__(self, last_message_id):
        '''
        Constructur for heartbeat messages

        :param last_message_id: the message id being acknowledged by this
            heartbeat message
        '''
        self.last_message_id = last_message_id

    @staticmethod
    def from_list(l):
        '''
        Parse list into a heartbeat message

        :param l: list to be parsed
        '''
        return Heartbeat(l[1])

    def to_list(self):
        '''
        Serialize this message into a list
        '''
        return [self.code, self.last_message_id]


class Notification(Message):
    '''
    Notification message
    '''
    code = 1

    def __init__(self, message_id, method, data=None):
        '''
        Constructor for notification messages

        :param method: name of the method being called
        :param data: data for the method being called
        :param message_id: id of this message
        '''
        self.message_id = message_id
        self.method = method
        self.data = data

    @staticmethod
    def from_list(l):
        '''
        Parse list into a notification message

        :param l: list to be parsed
        '''
        return Notification(*l[1:])

    def to_list(self):
        '''
        Serialize this message into a list
        '''
        return [self.code, self.message_id, self.method, self.data]


class Request(Message):
    '''
    Request message
    '''
    code = 2

    def __init__(self, message_id, method, data=None):
        '''
        Constructor for request messages

        :param method: name of the method being called
        :param data: data for the method being called
        :param message_id: id of this message
        '''
        self.message_id = message_id
        self.method = method
        self.data = data
        self._response = AsyncResult()

    def to_list(self):
        return [self.code, self.message_id, self.method, self.data]

    @staticmethod
    def from_list(l):
        '''
        Parse list into a request message

        :param l: list to be parsed
        '''
        return Request(*l[1:])

    @property
    def response(self):
        return self.get_response()

    @response.setter
    def response(self, response):
        self._response.set(response)

    def get_response(self, block=True, timeout=None):
        '''
        Get response for this request.

        :param block: block until response is available
        :type block: bool
        :param timeout: seconds to wait before raising a :class:`mushroom.rpc.RequestTimeout` error
        :type timeout: int or None
        :rtype: :class:`mushroom.rpc.Response` or :class:`mushroom.rpc.Error`
        :raises: :class:`mushroom.rpc.RequestTimeout`
        '''
        return self._response.get(block=block, timeout=timeout)


class Response(Message):
    '''
    Response message
    '''
    code = 3

    def __init__(self, message_id, request_message_id, data=None):
        '''
        Constructor for response messages

        :param request_message_id: the message id of the request which
                caused this response
        :param data: response data of the method which was called
        :param message_id: id of this message
        '''
        self.request_message_id = request_message_id
        self.message_id = message_id
        self.data = data
        self.request = None

    @staticmethod
    def for_request(message_id, request, data=None):
        '''
        Named constructor when the request is known. Some transports
        need the reference to the original request object when sending
        the reply for a request. Therefore the Engine_ generates
        all responses using this method.

        :param request: the request which caused this response
        :param data: response data of the method which was called
        :param message_id: id of this message
        '''
        response = Response(message_id, request.message_id, data)
        response.request = request
        return response

    @staticmethod
    def from_list(l):
        '''
        Parse list into a response message

        :param l: list to be parsed
        '''
        return Response(*l[1:])

    def to_list(self):
        '''
        Serialize this message into a list
        '''
        if isinstance(self.request, Request):
            return [self.code, self.message_id, self.request.message_id, self.data]
        else:
            return [self.code, self.message_id, self.request, self.data]


class Error(Message):
    '''
    Error message

    This is the message class and not the exception. The `RpcEngine`
    will raise a `RequestException` upon receiving this message type.
    '''
    code = 4

    def __init__(self, message_id, request_message_id, data=None):
        '''
        Constructor for error messages

        :param request_message_id: the message id of the request which
                caused this response
        :param data: response data of the method which was called
        :param message_id: id of this message
        '''
        self.request_message_id = request_message_id
        self.message_id = message_id
        self.data = data
        self.request = None

    @staticmethod
    def for_request(message_id, request, data=None):
        '''
        Named constructor when the request is known. Some transports
        need the reference to the original request object when sending
        the reply for a request. Therefore the `RpcEngine` generates
        all errors using this method.

        :param request: the request which caused this response
        :param data: response data of the method which was called
        :param message_id: id of this message
        '''
        error = Error(message_id, request.message_id, data)
        error.request = request
        return error

    @staticmethod
    def from_list(l):
        '''
        Parse list into a error message

        :param l: list to be parsed
        '''
        return Error(*l[1:])

    def to_list(self):
        '''
        Serialize this message into a list
        '''
        if isinstance(self.request, Request):
            return [self.code, self.message_id, self.request.message_id, self.data]
        else:
            return [self.code, self.message_id, self.request, self.data]


class Disconnect(Message):
    '''
    Disconnect message
    '''
    code = -1

    @staticmethod
    def from_list(l):
        '''
        Parse list into a disconnect message

        :param l: list to be parsed
        '''
        return Disconnect()

    def to_list(self):
        '''
        Serialize this message into a list
        '''
        return [self.code]


# Dictionary which maps the message codes to the according classes.
# This is only used by the `Message` class in the static `from_list`
# method.
MESSAGE_CLASS_BY_CODE = {
    Heartbeat.code: Heartbeat,
    Notification.code: Notification,
    Request.code: Request,
    Response.code: Response,
    Error.code: Error,
    Disconnect.code: Disconnect
}



from .exceptions import RequestValidationError


class Method(object):
    request_serializer = None
    response_serializer = None

    @classmethod
    def as_function(cls):
        def f(request):
            obj = cls()
            return obj(request)
        return f

    def __call__(self, request):
        self.request = request
        self.request_data = self.deserialize_request()
        self.response_data = self.call()
        self.response = self.serialize_response()
        return self.response

    def deserialize_request(self):
        if self.request_serializer:
            serializer = self.request_serializer(data=self.request.data)
            if not serializer.is_valid():
                raise RequestValidationError(serializer.errors)
            return serializer.validated_data
        else:
            return self.request.data

    def serialize_response(self, many=False):
        if self.response_serializer:
            return self.response_serializer(self.response_data, many=many).data
        else:
            return self.response_data


class ListMethod(Method):

    def call(self):
        return self.get_objects()

    def get_objects(self):
        return []


class PaginatedListMethod(ListMethod):
    paginator_min_limit = 1
    paginator_max_limit = None
    paginator_default_limit = None
    paginator_max_offset = None

    def call(self):
        self.paginator_offset = self.request.data.get('offset', 0)
        if isinstance(self.paginator_offset, int):
            if self.paginator_offset < 0:
                # FIXME raise a RequestValidationError?
                self.paginator_offset = 0
        else:
            # FIXME raise a RequestValidationError?
            self.paginator_offset = self.paginator_default_limit
        self.paginator_limit = self.request.data.get('limit', None)
        if isinstance(self.paginator_limit, int):
            if self.paginator_min_limit and self.paginator_limit < self.paginator_min_limit:
                self.paginator_limit = self.paginator_min_limit
            elif self.paginator_max_limit and self.paginator_limit > self.paginator_max_limit:
                self.paginator_limit = self.paginator_max_limit
        else:
            if self.paginator_limit is None:
                self.paginator_limit = self.paginator_max_limit
            else:
                # FIXME raise a RequestValidationError?
                self.paginator_limit = None
        self.all_objects = self.get_objects()
        self.object_count = self.get_object_count()
        self.objects = self.apply_pagination()
        return self.objects

    def get_object_count(self):
        return len(self.all_objects)

    def apply_pagination(self):
        if self.paginator_offset or self.paginator_limit:
            if self.paginator_limit:
                return self.all_objects[:self.paginator_limit]
            else:
                return self.all_objects[self.paginator_offset:self.paginator_offset+self.paginator_limit]
        return self.all_objects

    def serialize_response(self):
        return {
            'list': super(PaginatedListMethod, self).serialize_response(many=True),
            'count': self.object_count
        }



import sys

from django.core.management.base import BaseCommand, CommandError


class RunserverCommand(BaseCommand):
    '''
    Base class for creating management commands that start a mushroom server.
    In order to use this class the ``server_class`` attribute must be set.

    Example:

        ``myapp/management/commands/runmyserver.py``::

            import gevent.monkey
            from mushroom.django_support import RunserverCommand

            from myapp.server import MyServer

            class Command(RunserverCommand):
                server_class = MyServer

                def get_server(self, listener, **options):
                    gevent.monkey.patch_all()
                    return super(Command, self).get_server(listener, **options)

        ``myapp/server.py``::

            from mushroom.simple import SimpleServer

            class MyServer(SimpleServer):

                def rpc_ping(self, data):
                    return 'pong'

        Add ``myapp`` to ``settings.INSTALLED_APPS`` and do not forget
        to create the neccesary ``__init__.py`` files inside the directories
        so python can load the modules. The command can now be used like
        any other Django management command: ``manage.py runmyserver``
    '''
    args = '[optional port number, or ipaddr:port]'
    server_class = None
    server_name = 'mushroom server'
    default_host = '127.0.0.1'
    default_port = 39288 # picked by random.randint(1024, 65535)

    @property
    def help(cls):
        '''
        Property which creates a dynamic help text including the
        ``server_name`` and ``server_class``.
        '''
        return "Start %(server_name)s (server_class=%(server_module)s.%(server_class)s" % {
            'server_name': cls.server_name,
            'server_module': cls.server_class.__module__,
            'server_class': cls.server_class.__name__,
        }

    def handle(self, *args, **options):
        command_name = self.__class__.__module__.rsplit('.')[-1]
        if len(args) > 1:
            raise CommandError('Usage is %s [optional port number, or ipaddr:port]' % command_name)
        if len(args) == 0:
            listener = (self.default_host, self.default_port)
        else:
            try:
                if ':' in args[0]:
                    listener = args[0].split(':', 1)
                    listener = (listener[0], int(listener[1]))
                else:
                    listener = (self.default_host, int(args[0]))
            except ValueError:
                raise CommandError('%r is not a valid port number or address:port pair.' % args[0])
        self.stdout.write((
            'Starting %(server_name)s at %(url)s\n'
            'Quit the server with %(quit_command)s.\n'
        ) % {
            'url': 'http://%s:%d/' % listener,
            'server_name': self.server_name,
            'quit_command': 'CTRL-BREAK' if sys.platform == 'win32' else 'CONTROL-C'
        })
        server = self.get_server(listener, **options)
        server.run()

    def get_server(self, listener, **options):
        '''
        This method is creates the actual server object. It can be
        overwritten to monkey patch things using `gevent.monkey` or
        perform other setup tasks prior creating the server object.
        '''
        return self.server_class(listener)





try:
    import simplejson as json
except ImportError:
    import json

from gevent.event import Event
from gevent.lock import Semaphore
from geventwebsocket.websocket import WebSocketError

from mushroom.rpc import Disconnect
from mushroom.rpc import Message
from mushroom.rpc import Heartbeat
from mushroom.transport import UnreliableTransport


class HttpError(RuntimeError):
    code = ''
    message = ''
    headers = {}

    def __init__(self, message=None):
        self.message = message or self.message
        super(HttpError, self).__init__(self.message)


class HttpUnauthorized(HttpError):
    code = '401 Unauthorized'
    def __init__(self, auth_scheme=None):
        if not auth_scheme:
            auth_scheme = 'Mushroom'
        self.headers = [
            ('WWW-Authenticate', auth_scheme)
        ]
        super(HttpUnauthorized, self).__init__()


class HttpNotFound(HttpError):
    code = '404 Not Found'


class HttpMethodNotAllowed(HttpError):
    code = '405 Method Not Allowed'
    def __init__(self, allowed_methods):
        self.headers = [
            ('Allow', ', '.join(method.upper() for method in allowed_methods))
        ]
        super(HttpMethodNotAllowed, self).__init__()


class HttpInternalServerError(HttpError):
    code = '500 Internal Server Error'


class HttpNotImplemented(HttpError):
    code = '501 Not Implemented'


class HttpRequest(object):

    def __init__(self, environ):
        # FIXME add validation
        self.environ = environ
        self.method = environ['REQUEST_METHOD']
        self.raw_data = environ['wsgi.input'].read()
        if self.raw_data:
            self.data = json.loads(self.raw_data)
        else:
            self.data = None
        # Convert path to list of atoms
        self.path = [p for p in environ['PATH_INFO'].split('/') if p]


class HttpResponse(object):

    def __init__(self, code='200 OK', content='', extra_headers=None):
        self.code = code
        self.content = content
        self.headers = {
            'Access-Control-Allow-Origin': '*',
            'Content-Type': 'text/plain',
        }
        self.headers.update(extra_headers or {})


class JsonResponse(HttpResponse):

    def __init__(self, data):
        super(JsonResponse, self).__init__(
            content=json.dumps(data),
            extra_headers={
                'Content-Type': 'application/json',
            }
        )


class HttpTransport(UnreliableTransport):

    def get_url(self, protocol, request, session):
        host = request.environ['HTTP_HOST']
        path = request.environ['PATH_INFO'].rstrip('/')
        return '%s://%s%s/%s/' % (protocol, host, path, session.id)

    def handle_http_request(self, request, session):
        self.remote_addr = request.environ.get('REMOTE_ADDR', None)


class PollTransport(HttpTransport):

    name = 'poll'
    timeout = 40

    def __init__(self):
        super(PollTransport, self).__init__()
        self.messages_ready = Event()

    def get_handshake_data(self, request, session):
        protocol = request.environ.get('wsgi.url_scheme', 'http')
        return {
            'transport': self.name,
            'url': self.get_url(protocol, request, session)
        }

    def handle_http_request(self, request, session):
        super(PollTransport, self).handle_http_request(request, session)
        # Only allow POST requests for polling
        if request.method != 'POST':
            raise HttpMethodNotAllowed(['POST'])
        assert isinstance(request.data, list)
        heartbeat = None
        for message_data in request.data:
            message = Message.from_list(message_data)
            message.session = session
            self.handle_message(message)
            if isinstance(message, Heartbeat):
                heartbeat = message
        if heartbeat:
            self.handle_connect()
            self.messages_ready.wait(self.timeout)
            self.messages_ready.clear()
            self.handle_disconnect(reconnect=True)
            frames = [message.to_list() for message in self.messages]
            if self.last_message_id is not None:
                frames.append(Heartbeat(self.last_message_id).to_list())
            return JsonResponse(frames)
        else:
            return JsonResponse(None)

    def handle_disconnect(self, reconnect=False):
        if not reconnect:
            self.send(Disconnect())
        super(PollTransport, self).handle_disconnect(reconnect=reconnect)

    def real_send(self, message):
        self.messages_ready.set()


class WebSocketTransport(HttpTransport):

    name = 'ws'

    def __init__(self):
        super(WebSocketTransport, self).__init__()
        self.send_mutex = Semaphore()
        self.ws = None

    def get_handshake_data(self, request, session):
        if hasattr(request, 'is_secure'):
            # Use request.is_secure from Django WSGIRequest objects
            protocol = 'wss' if request.is_secure() else 'ws'
        else:
            if request.environ.get('wsgi.url_scheme') == 'https':
                protocol = 'wss'
            else:
                protocol = 'ws'
        return {
            'transport': self.name,
            'url': self.get_url(protocol, request, session)
        }

    def handle_http_request(self, request, session):
        super(WebSocketTransport, self).handle_http_request(request, session)
        assert not self.ws
        self.ws = request.environ['wsgi.websocket']
        self.handle_connect()
        # Process incoming messages
        try:
            while self.ws is not None:
                frame = self.ws.receive()
                if frame is None:
                    # Disconnect
                    return
                message_data = json.loads(frame)
                message = Message.from_list(message_data)
                message.session = session
                self.handle_message(message)
        finally:
            if self.state() == 'CONNECTED':
                # WebSocket was closed unexpectedly.
                self.handle_disconnect()

    def handle_message(self, message):
        super(WebSocketTransport, self).handle_message(message)
        if hasattr(message, 'message_id'):
            self.real_send(Heartbeat(self.last_message_id))

    def handle_disconnect(self):
        self.ws = None
        super(WebSocketTransport, self).handle_disconnect()

    def real_send(self, message):
        message_data = message.to_list()
        frame = json.dumps(message_data)
        with self.send_mutex:
            if self.ws:
                try:
                    self.ws.send(frame)
                except WebSocketError:
                    self.handle_disconnect()



#!/usr/bin/env python

import example_pythonpath
from example_utils import ExampleServer, ExampleStaticFile


class ClientApiServer(ExampleServer):
    urls = [
        ('/', ExampleStaticFile('client-api.html')),
    ] + ExampleServer.urls


if __name__ == '__main__':
    ClientApiServer.main()



from itertools import count
import logging

import gevent

from . import signals
from .exceptions import RpcError, RequestTimeout, RequestException, \
        MethodNotFound
from .messages import Request, Notification, Response, Error

logger = logging.getLogger('mushroom.rpc')


class Engine(object):
    '''
    Transport neutral message factory and mapper between requests and
    responses. This is the heart of all RPC handling.
    '''

    def __init__(self, transport, rpc_handler):
        # Transport for sending and receiving messages
        self.transport = transport
        # Bind transport to the engine
        assert transport.rpc_engine is None, \
                'transport already bound to another rpc engine'
        transport.rpc_engine = self
        # Handler for inbound requests and notifications
        self.rpc_handler = rpc_handler
        # Generator for outbount message ids
        self.message_id_generator = count()
        # Dictionary for mapping outbound requests to inbound responses
        self.requests = {}

    def next_message_id(self):
        '''
        Generate the next message id for outbound messages.

        Returns: the next message id
        '''
        return next(self.message_id_generator)

    def notify(self, method, data=None, **kwargs):
        '''
        Send a notification.

        :param method: name of the method to be called
        :param data: data for the method being called
        :param kwargs: transport specific arguments
        '''
        message = Notification(self.next_message_id(), method, data)
        self.send(message, **kwargs)

    def request(self, method, data=None, timeout=None, **kwargs):
        '''
        Send a request and wait for the response or timeout. If no response
        for the given method is received within `timeout` seconds a
        `RequestTimeout` exception is raised.

        :param method: name of the method to be called
        :param data: data for the method being called
        :param timeout: timeout in seconds for this request
        :param kwargs: transport specific arguments
        '''
        request = Request(self.next_message_id(), method, data)
        self.requests[request.message_id] = request
        self.send(request, **kwargs)
        try:
            response = request.get_response(timeout=timeout)
        except gevent.Timeout:
            raise RequestTimeout
        if isinstance(response, Response):
            return response.data
        elif isinstance(response, Error):
            raise RequestException(response.data)
        else:
            raise RuntimeError('Unexpected type of response: %s' % type(response))

    def send(self, message, **kwargs):
        '''
        Hand message over to the transport.

        :param message: message to be sent
        :param kwargs: transport specific arguments
        '''
        self.transport.send(message, **kwargs)

    def handle_message(self, message):
        '''
        Handle message received from the transport.

        :param message: message to be handled
        '''
        if isinstance(message, Notification):
            # Spawn worker to process the notification. The response of
            # the worker is ignored.
            @signals.message_handler
            def worker():
                try:
                    self.rpc_handler(message)
                except MethodNotFound as e:
                    logger.warning('MethodNotFound: %s' % e.method_name)
                except RpcError as e:
                    logger.debug(e, exc_info=True)
                except Exception as e:
                    logger.exception(e)
            gevent.spawn(worker)
        elif isinstance(message, Request):
            # Spawn worker which waits for the response of the rpc handler
            # and sends the response message.
            @signals.message_handler
            def worker():
                try:
                    response = self.rpc_handler(message)
                except MethodNotFound as e:
                    logger.warning('MethodNotFound: %s' % e.method_name)
                    self.send(Error.for_request(self.next_message_id(),
                        message, e.method_name))
                except RpcError as e:
                    logger.debug(e, exc_info=True)
                    self.send(Error.for_request(self.next_message_id(),
                        message, e.message))
                except Exception as e:
                    logger.exception(e)
                    self.send(Error.for_request(self.next_message_id(),
                        message, 'Internal server error'))
                else:
                    if not isinstance(response, Response):
                        response = Response.for_request(self.next_message_id(),
                                message, response)
                    self.send(response)
            gevent.spawn(worker)
        elif isinstance(message, (Response, Error)):
            # Find request according to response or log an error.
            try:
                message.request = self.requests.pop(message.request_message_id)
            except KeyError:
                logger.error('Response for unknown request message id: %r' %
                        message.request_message_id)
                return
            message.request.response = message
        else:
            # Heartbeats and Disconnect messages are handled by the
            # transport and are never passed to the RPC engine.
            raise RuntimeError('Unsupported message type: %s' % type(message))



#!/usr/bin/env python

import os
from setuptools import setup

def read(*p):
    '''Utility function to read files relative to the project root'''
    return open(os.path.join(os.path.dirname(__file__), *p)).read()

def get_version():
    import re
    '''Get __version__ information from __init__.py without importing it'''
    VERSION_RE = r'^__version__\s*=\s*[\'"]([^\'"]+)[\'"]'
    VERSION_PATTERN = re.compile(VERSION_RE, re.MULTILINE)
    m = VERSION_PATTERN.search(read('mushroom', '__init__.py'))
    if m:
        return m.group(1)
    else:
        raise RuntimeError('Could not get __version__ from mushroom/__init__.py')

# Prevent "TypeError: 'NoneType' object is not callable" when running tests.
# (http://www.eby-sarna.com/pipermail/peak/2010-May/003357.html)
try:
    import multiprocessing
except ImportError:
    pass

setup(
    name='mushroom',
    version=get_version(),
    description='real-time web messaging',
    long_description=read('README'),
    author='Michael P. Jung',
    author_email='michael.jung@terreon.de',
    license='BSD',
    keywords='http push framework mushroom magic',
    url='https://bitbucket.org/terreon/mushroom',
    packages=[
        'mushroom',
        'mushroom.rpc',
        'mushroom.django',
        'mushroom.gunicorn'
    ],
    test_suite='mushroom.tests',
    setup_requires=[
        'kombu',
        'selenium',
    ],
    install_requires=[
        'gevent',
        'gevent-websocket',
        'six',
    ],
    extras_require={
        # Kombu is optional and only required if you want to use mushroom
        # for communication with a messaging system like RabbitMQ.
        'messaging': [
            'kombu',
        ]
    },
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.6',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ]
)
